/*
 * SquidPCB.java
 *
 * Copyright 2007, 2008 by Fernando Lemes da Silva.
 * All rights reserved.
 * 
 * This file is part of SquidPCB project.
 * 
 * SquidPCB project is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SquidPCB project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with the SquidPCB package. If not, see <http://www.gnu.org/licenses/>.
 * 
 * [Begin of academic info]
 * 
 *     Instituto de Matemática e Estatística (IME-USP)
 *     MAC0499 - Trabalho de formatura supervisionado
 * 
 *     Projeto: 		SquidPCB - Squid-cache Pornography Content Blocker
 *     Aluno: 			Fernando Lemes da Silva (Nr.USP 2371843)
 *     Orientado por: 	Prof. Dr. Roberto Hirata Jr.
 *     Contato:			lemes at linux.ime.usp.br / ferlemes at gmail dot com
 *     Data:			28 de dezembro de 2007
 *     
 * [End of academic info]
 */

package br.usp.ime.linux.lemes;

import br.usp.ime.linux.lemes.Datastructure.DataStructureManager;
import br.usp.ime.linux.lemes.IPC.ConnectionServer;
import br.usp.ime.linux.lemes.IPC.UserAuthenticator;
import br.usp.ime.linux.lemes.WordSearch.WordSearch;

/**
 * This class is used to create the SquidPCB server with all the data
 * structures and connection server.
 * 
 * @author		Fernando Lemes da Silva
 * @version		0.1b (stable, but needing some improvement)
 */
public class SquidPCB {
	
	// Server is available for some methods
	private static ConnectionServer localServer;
	
	/**
	 * Method used to register the configuration options. It uses some static
	 * methods of the ConfigurationParser class to define witch options are
	 * used for this class.
	 * 
	 * @see br.usp.ime.linux.lemes.ConfigurationParser
	 */
	public static void registerConfigs() {
		
		ConfigurationParser.setOptionName("logfile",	"squidpcb.log");
		ConfigurationParser.setOptionName("config",		"squidpcb.cfg");
		ConfigurationParser.setOptionName("loglevel",	"3");
		ConfigurationParser.setOptionName("verbose",	"2");
		
		ConfigurationParser.setOptionNameCommandlineAlias("logfile",	"-L");
		ConfigurationParser.setOptionNameCommandlineAlias("config",		"-c");
		ConfigurationParser.setOptionNameCommandlineAlias("loglevel",	"-l");
		ConfigurationParser.setOptionNameCommandlineAlias("verbose",	"-v");
		
		ConfigurationParser.setOptionName("text_badwords_file");
		ConfigurationParser.setOptionName("url_badwords_file");
		ConfigurationParser.setOptionName("limit_porn_probability");
	    
	}
	
	public static void shutdownServer() {
		SquidPCB.localServer.shutdown();
	}

	/**
	 * The main loop for this class. In this block all the server data structures
	 * are created and a TCP server is instantiated to accept connections.
	 * 
	 * @param options The command line arguments.
	 */
	public static void main(String commandLineOptions[]) {

		// Software announce and license info
		System.out.println("SquidPCB v0.1 - Copyright (C) 2007, 2008 Fernando Lemes da Silva (http://squidpcb.sf.net)\n" +
				"This program comes with ABSOLUTELY NO WARRANTY. This is free software,\n" +
				"and you are welcome to redistribute it under certain conditions.\n" +
				"Please read the LICENSE.TXT file for details or get the license at\n" +
				"http://www.gnu.org/licenses/gpl.txt\n");
		
		// Initialize ConfigurationParser and basic configs
		ConfigurationParser.initialize();
		ConfigurationParser.parseCommandLine(commandLineOptions);
		
		// Logging initialization
		Logging.setLogfile(ConfigurationParser.getStringValue("logfile"));
		Logging.setFileLogLevel(ConfigurationParser.getIntValue("loglevel"));
		Logging.setScreenLogLevel(ConfigurationParser.getIntValue("verbose"));
		
		// Read configs from file
        if (!ConfigurationParser.readFile(ConfigurationParser.getStringValue("config")))
        		Logging.panic("SquidPCB: Config file not found!");
        
        // Initialize the UserAuthenticator
        UserAuthenticator.initialize();
        
        // Create WordSearch object for URL analisys
        WordSearch 	urlBadwords = new WordSearch();
        if (ConfigurationParser.isDefined("url_badwords_file"))
        	urlBadwords = new WordSearch(
        		ConfigurationParser.getStringValue("url_badwords_file"));
        
        // Create WordSearch object for text objects analysis
        WordSearch	textBadwords = new WordSearch();
        if (ConfigurationParser.isDefined("text_badwords_file"))
        	textBadwords = new WordSearch(
        		ConfigurationParser.getStringValue("text_badwords_file"));
        		        
        // Url Datastructure - "The main object" of this server
        DataStructureManager	dsManager		= new DataStructureManager();
        
        // Lets start the server
        Logging.info("SquidPCB: Starting server...");
        
        SquidPCB.localServer = new ConnectionServer(dsManager, urlBadwords, textBadwords);
        SquidPCB.localServer.start();
        
        /*
         * TODO: A new thread must be instantiated to perform the dsManager maintance,
         * like swappping, saving to file (to keep a backup of all the constructed tree),
         * and 'updating' (to be reviewed). 
         */
        
        // Lets start the server
        Logging.info("SquidPCB: Shutting down server...");
        
	}

}
